;File "MAIN-PRG" data

;After loading the boot files, $0102 is set to $35 which makes so that the
;$DFFC vector (disk Reset vector) is used and the BIOS is skipped.

;$0103 indicate reset type: $AC = first boot of the game,
;$53 = game was soft-reseted by the user.

;To use your own IRQ routine, you must manually write $C0 to $0101.

;There are 3 possible NMI vectors, NMI #3 is used by default.

;On boot, scroll/mirroring setting is set to V-Scroll (horizontal mirroring),
;the stack pointer is $FF, and the I flag is clear. System RAM is filled with
;values used by the BIOS, and PRG RAM is uninitialized, except for parts of it
;which has files loaded in.

.include "constants.asm"

.segment "HEADER";File Header Block
  .byte $03 ;Block code
  .byte $01 ;File Number
  .byte $01 ;File ID
  .byte "MAIN-PRG" ;File Name (8 upercase ASCII characters)
  .word FDS_PRAM+$4000 ;Where to load file to (16-bit little endian)
  ;Since it's a small program we load it to $A000 instead of the beginning
  ;of PRAM.
  .word (MainPRGEnd - MainPRGStart) ;File Size (16-bit little endian)
  .byte $00 ;File type (PRG)
  ;0:Program (PRAM)
  ;1:Character (CRAM)
  ;2:Nametable (VRAM)
  ;.byte $00,$00 ;CRC - Not used in the .fds file format

;File Data Block
  .byte $04 ;Block code

  ;.base $A000
  .segment "CODE"


MainPRGStart:

;-------------------------------------------------------------------------------
NMI1:
  jsr FDS_HOOK
  rti
NMI2:
  jsr FDS_HOOK
  rti
NMI3:
  php ;backup processor status
  pha ;backup A,X and Y
  txa
  pha
  tya
  pha
  ;NMI code goes here
  
  jsr FDS_HOOK
  pla ;restore A, X and Y
  tay
  pla
  tax
  pla
  plp ;restore processor status  
  rti
  
RESET:
  ;Basic init code
  sei
  cld
  ldx #$40
  stx $4017 ; disable APU frame IRQ
  ldx #$00
  stx $2000 ;disable NMI
  stx $2001 ;disable rendering  
  
  jsr FDS_HOOK
  
  jsr init_palette
  ldx #$20 ;pick nametable 0
  jsr init_nametable ;expects high nametable address in X
  ldx #$24 ;pick nametable 1
  jsr init_nametable
  ldx #$23 ;pick nametable 0 attributes
  jsr init_attribute ;expects high attribute table address in X
  ldx #$27 ;pick nametable 1 attributes
  jsr init_attribute

  
  ;Setup Sound
  lda #%00000001 ;enable Pulse 1
  sta $4015
  
  ;Pulse 1
  lda #%10111111 ;Duty 10 (50%), Length Counter disabled,
  ;Saw Envelopes disabled, Volume F (loudest)
  sta $4000
  lda #$C9 ;$0C9 is a C# note (NTSC)
  sta $4002
  lda #$00
  sta $4003

  ;lda #%00011000
  ;sta $2001 ;turn on rendering

Main: ;main program eternal loop
  jmp Main

  
;Subroutines:

FDS_HOOK:
;Prevent the program to suddenly reset to
;the FDS menu on NMI, IRQ, or RESET.
  pha
  lda #$C0
  sta $0100 ;set NMI
  lda #$80
  sta $0101 ;set IRQ
  lda #$35
  sta $0102 ;set RESET vector
  lda #$53
  sta $0103 ;set boot type to "reset by user"
  pla
  rts
  
init_palette:
  lda $2002 ;read $2002 to reset high/low latch to high
  lda #$3F
  sta $2006 ;write the high byte of $3F00
  lda #$00
  sta $2006 ;write the low byte of $3F00
  ldx #$00 ;loop counter
@loop:
  lda Palette,x ;load data from Palette + X
  sta $2007 ;write data to PPU $3F00, $3F01 etc
  inx
  cpx #$20 ;compare X to $20 (decimal 32) for all palette entries to be filled
  bne @loop
  rts

init_nametable:
;Clears nametable by filling it with white space characters ($20).
;Takes high byte of nametable address as an argument in X.
;X = $20 for nametable0.
;X = $24 for nametable1.
;X = $28 for nametable2.
;X = $2C for nametable3.
  lda $2002 ;reset high/low latch
  stx $2006 ;write the high byte of nametable address
  lda #$00
  sta $2006 ;write the low byte of nametable address
  lda #$2A ;fill nametable with white spaces " " to clear it
  ldy #$1E ;30 rows
:
  ldx #$20 ;32 columns
:
  sta $2007
  dex
  bne :-
  dey
  bne :--
  rts

init_attribute:
;Clears the attributes of a nametable by filling it with zeroes.
;Takes high byte of nametable address as an argument in X.
;X = $23 for nametable0 attributes.
;X = $27 for nametable1 attributes.
;X = $2B for nametable2 attributes.
;X = $2F for nametable3 attributes.
  lda $2002 ;reset high/low latch
  stx $2006 ;write high byte of address
  lda #$C0
  sta $2006 ;write low byte of address
  ldx #$40 ;64 attributes
  lda #$00 ;set all attributes to 0
@loop:
  sta $2007 ;write to PPU
  dex
  bne @loop
  rts

;Data:
Palette:
  .byte $0F,$30,$21,$02,  $0F,$30,$27,$18,  $0F,$30,$27,$16,  $0F,$29,$36,$17
  .byte $0F,$36,$16,$02,  $0F,$27,$30,$19,  $0F,$35,$25,$17,  $0F,$30,$27,$16

;-------------------------------------------------------------------------------
IRQ:
  jsr FDS_HOOK
  rti

;===============================================================================
; Interrupt Vector Table
;===============================================================================
  .segment "VECTORS"
  .word NMI1
  .word NMI2
  .word NMI3
  .word RESET
  .word IRQ

;-------------------------------------------------------------------------------

MainPRGEnd:
